import io
from test import POFILE_START

import pytest

from potojson.__main__ import run


def test_stdin(capsys, monkeypatch):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr ""\n'
    monkeypatch.setattr("sys.stdin", io.StringIO(pofile_content))

    expected_output = '{"Hello": ""}\n'
    output, exitcode = run()
    out, err = capsys.readouterr()

    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


def test_filepath(capsys, tmp_file):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr ""\n'
    expected_output = '{"Hello": ""}\n'

    with tmp_file(pofile_content, ".po") as po_filepath:
        output, exitcode = run([po_filepath])
        out, err = capsys.readouterr()

    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-m", "--fallback-to-msgid"])
def test_fallback_to_msgid(capsys, arg):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr ""\n'
    output, exitcode = run([pofile_content, arg])
    out, err = capsys.readouterr()

    expected_output = '{"Hello": "Hello"}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-f", "--fuzzy"])
def test_fuzzy(capsys, arg):
    pofile_content = POFILE_START + '#, fuzzy\nmsgid "Hello"\nmsgstr "Hola"\n'
    output, exitcode = run([pofile_content, arg])
    out, err = capsys.readouterr()

    expected_output = '{"Hello": "Hola"}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-p", "--pretty"])
def test_pretty(capsys, arg):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr "Hola"\n'
    output, exitcode = run([pofile_content, arg])
    out, err = capsys.readouterr()

    expected_output = '{\n  "Hello": "Hola"\n}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-i", "--indent"])
def test_indent(capsys, arg):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr "Hola"\n'
    output, exitcode = run([pofile_content, arg, "3", "-p"])
    out, err = capsys.readouterr()

    expected_output = '{\n   "Hello": "Hola"\n}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-i", "--indent"])
def test_indent_no_pretty(capsys, arg):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr "Hola"\n'
    output, exitcode = run([pofile_content, arg, "3"])
    out, err = capsys.readouterr()

    # without pretty doesn't work
    expected_output = '{"Hello": "Hola"}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-l", "--language"])
def test_language(capsys, arg):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr "Hola"\n'
    output, exitcode = run([pofile_content, arg, "es"])
    out, err = capsys.readouterr()

    expected_output = '{"Hello": "Hola", "": {"language": "es"}}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-s", "--plural-forms"])
def test_plural_forms(capsys, arg):
    pofile_content = POFILE_START + 'msgid "Hello"\nmsgstr "Hola"\n'
    output, exitcode = run([pofile_content, arg, "nplurals=2; plural=n != 1;"])
    out, err = capsys.readouterr()

    expected_output = (
        '{"Hello": "Hola", "": {"plural-forms": "nplurals=2; plural=n != 1;"}}\n'
    )
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output


@pytest.mark.parametrize("arg", ["-k", "--sort-keys"])
def test_sort_keys(capsys, arg):
    pofile_content = (
        POFILE_START + 'msgid "Hello"\nmsgstr "Hola"\nmsgid "A"\nmsgstr "B"\n'
    )
    output, exitcode = run([pofile_content, arg])
    out, err = capsys.readouterr()

    expected_output = '{"A": "B", "Hello": "Hola"}\n'
    assert exitcode == 0
    assert output == expected_output
    assert out == expected_output



#!/usr/bin/env python

"""potojson command line interface."""

import argparse
import sys

from potojson import __description__, __version__, pofile_to_json


def build_parser():
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version="%(prog)s " + __version__,
        help="Show program version number and exit.",
    )
    parser.add_argument(
        "po_filepath_or_content",
        nargs="*",
        metavar="PO_FILEPATH_OR_CONTENT",
        help="Path to pofile or pofile content as string."
        " If not provided, will be read from STDIN.",
    )
    parser.add_argument(
        "-m",
        "--fallback-to-msgid",
        dest="fallback_to_msgid",
        action="store_true",
        help="Use msgid if translation is missing.",
    )
    parser.add_argument(
        "-f",
        "--fuzzy",
        dest="fuzzy",
        action="store_true",
        help="Include fuzzy messages.",
    )
    parser.add_argument(
        "-p",
        "--pretty",
        dest="pretty",
        action="store_true",
        help="Pretty-print JSON output.",
    )
    parser.add_argument(
        "-i",
        "--indent",
        dest="indent",
        help="Number of spaces for indentation used pretty-"
        "printing JSON output. Only takes effect passing"
        " '--fuzzy' option.",
        metavar="N",
        type=int,
    )
    parser.add_argument(
        "-l",
        "--language",
        dest="language",
        default=None,
        help="Language for the translations. Will be inserted"
        " in the empty key of the JSON output. If not"
        " provided and the passed pofile includes the"
        ' "Language" header, will be extrated from it.',
    )
    parser.add_argument(
        "-s",
        "--plural-forms",
        dest="plural_forms",
        default=None,
        help="Plural forms for the language of the"
        " translations. Will be insertedin the empty key"
        " of the JSON output. If not provided and the"
        ' passed pofile includes the "Plural-Forms"'
        " header, will be extrated from it.",
    )
    parser.add_argument(
        "-k",
        "--sort-keys",
        dest="sort_keys",
        action="store_true",
        help="Sort JSON output by keys.",
    )
    return parser


def parse_options(args=[]):
    parser = build_parser()
    if "-h" in args or "--help" in args:
        parser.print_help()
        sys.exit(0)
    opts = parser.parse_args(args)

    if not sys.stdin.isatty():
        opts.po_filepath_or_content = sys.stdin.read().strip("\n")
    elif isinstance(opts.po_filepath_or_content, list):
        opts.po_filepath_or_content = opts.po_filepath_or_content[0]

    return opts


def run(args=[]):
    opts = parse_options(args)

    kwargs = dict(
        fallback_to_msgid=opts.fallback_to_msgid,
        fuzzy=opts.fuzzy,
        pretty=opts.pretty,
        language=opts.language,
        plural_forms=opts.plural_forms,
        sort_keys=opts.sort_keys,
    )
    if isinstance(opts.indent, int):
        kwargs["indent"] = opts.indent

    output = "%s\n" % pofile_to_json(opts.po_filepath_or_content, **kwargs)
    sys.stdout.write(output)

    return (output, 0)


def main():
    sys.exit(run(args=sys.argv[1:])[1])


if __name__ == "__main__":
    main()
